@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2010-2022 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -18,17 +18,17 @@
 """Basic WSGI Application object for REST server."""
 
 import re
+import hmac
 import logging
 
 from base64 import b64decode
-from falcon import API, HTTPUnauthorized
+from falcon import App, HTTPUnauthorized
 from falcon.routing import map_http_methods, set_default_responders
 from mailman.config import config
-from mailman.database.transaction import transactional
+from mailman.database.transaction import api_transaction
+from mailman.rest.helpers import bad_request
 from mailman.rest.root import Root
 from public import public
-from wsgiref.simple_server import (
-    WSGIRequestHandler, WSGIServer, make_server as wsgi_server)
 
 
 log = logging.getLogger('mailman.http')
@@ -41,44 +41,6 @@ UTF8 = 'utf-8'
 WILDCARD_ACCEPT_HEADER = '*/*'
 
 
-class AdminWSGIServer(WSGIServer):
-    """Server class that integrates error handling with our log files."""
-
-    def handle_error(self, request, client_address):
-        # Interpose base class method so that the exception gets printed to
-        # our log file rather than stderr.
-        log.exception('REST server exception during request from %s',
-                      client_address)
-
-
-class StderrLogger:
-    def __init__(self):
-        self._buffer = []
-
-    def write(self, message):
-        self._buffer.append(message)
-
-    def flush(self):
-        self._buffer.insert(0, 'REST request handler error:\n')
-        log.error(EMPTYSTRING.join(self._buffer))
-        self._buffer = []
-
-
-class AdminWebServiceWSGIRequestHandler(WSGIRequestHandler):
-    """Handler class which just logs output to the right place."""
-
-    default_request_version = 'HTTP/1.1'
-
-    def log_message(self, format, *args):
-        """See `BaseHTTPRequestHandler`."""
-        log.info('%s - - %s', self.address_string(), format % args)
-
-    def get_stderr(self):
-        # Return a fake stderr object that will actually write its output to
-        # the log file.
-        return StderrLogger()
-
-
 class Middleware:
     """Falcon middleware object for Mailman's REST API.
 
@@ -94,7 +56,8 @@ class Middleware:
             credentials = b64decode(request.auth[6:]).decode('utf-8')
             username, password = credentials.split(':', 1)
             if (username == config.webservice.admin_user and
-                    password == config.webservice.admin_pass):
+                    hmac.compare_digest(
+                        password, config.webservice.admin_pass)):
                 authorized = True
         if not authorized:
             # Not authorized.
@@ -105,6 +68,20 @@ class Middleware:
                 challenges=[realm])
 
 
+def handle_ValueError(exc, request, response, params):
+    """Handle ValueErrors in API code to return HTTPBadRequest.
+
+    ValueErrors are raised often by Validator and should not return a 500 error
+    resposne to the client.  This is a stop-gap for 500 errors due to
+    ValueErrors, it is recommended that they be handled at the call-site,
+    instead of here.
+    """
+    # Only handle ValueError, raise anything else right back.
+    if not isinstance(exc, ValueError):
+        raise exc
+    bad_request(response, str(exc))
+
+
 class ObjectRouter:
     def __init__(self, root):
         self._root = root
@@ -196,7 +173,7 @@ class ObjectRouter:
                 return None, None, None
 
 
-class RootedAPI(API):
+class RootedAPI(App):
     def __init__(self, root, *args, **kws):
         super().__init__(
             *args,
@@ -213,7 +190,7 @@ class RootedAPI(API):
     # Override the base class implementation to wrap a transactional
     # handler around the call, so that the current transaction is
     # committed if no errors occur, and aborted otherwise.
-    @transactional
+    @api_transaction
     def __call__(self, environ, start_response):
         return super().__call__(environ, start_response)
 
@@ -221,19 +198,6 @@ class RootedAPI(API):
 @public
 def make_application():
     """Return a callable WSGI application object."""
-    return RootedAPI(Root())
-
-
-@public
-def make_server():
-    """Create the Mailman REST server.
-
-    Use this if you just want to run Mailman's wsgiref-based REST server.
-    """
-    host = config.webservice.hostname
-    port = int(config.webservice.port)
-    server = wsgi_server(
-        host, port, make_application(),
-        server_class=AdminWSGIServer,
-        handler_class=AdminWebServiceWSGIRequestHandler)
-    return server
+    app = RootedAPI(Root())
+    app.add_error_handler(ValueError, handle_ValueError)
+    return app
